/*******************************************************************************
********************************************************************************

	File	: main.c
	Project	: EQDRIVE
	Version	: 0.100
	Date	: 31.10.2011
	Author	: Ruslan Korop                           
	Company	: Home                            

MCU: STM32F100PB, board - STM32DISCOVERY.
Clock frequency	: 24,000000 MHz

********************************************************************************
*******************************************************************************/

#include    "main.h"

#define     LED3_ON		GPIO_BSRR_BS9
#define     LED3		GPIO_ODR_ODR9
#define     LED4		GPIO_ODR_ODR8
#define     LED3_OF		GPIO_BSRR_BR9
#define     LED4_ON		GPIO_BSRR_BS8
#define     LED4_OF		GPIO_BSRR_BR8

//Global variables
u32 SysTickTime;
//u32 PreDivTimerRA;
u8 FlagStepRA, FlagStepDEC, FlagRxEnd, FlagResetTimeOut;
u8 RxBuffer[10], TxBuffer[10], RxCounter, TxCounter, TxRdIndex;

struct RegisterAxis RA, DEC;


int main(void){
//u16 temp16;
static u32 TimeOut;
	
	//       RA  DEC
	//RA.Speed = 620;
	RA.Speed = 5728;
	RA.ControlMODE = 1;
	RA.StatusMODE = 1;
	DEC.Speed = 5728;
	DEC.ControlMODE = 1;
	DEC.StatusMODE = 1;
	
	init_clock(); //   
	init_peripherals(); //   
	//SysTick_Config(2400000);
	
	//     
	EN_RA_DW;
	REF_RA_UP;
	DIR_RA_UP;
	EN_DEC_DW;
	REF_DEC_UP;
	DIR_DEC_UP;
	
	printf ("EQDRIVE v0.100\r");
	printf ("EQMOD driver v0.100\r");
	
    while(1){ //  
		/*
		ADC1->CR2 |= ADC_CR2_SWSTART;
		while ((ADC1->SR & ADC_SR_EOC) != ADC_SR_EOC);
		temp16 = ADC1->DR;
		if (temp16 > 0x7FF) GPIOC->BSRR=LED4_ON;
		else GPIOC->BSRR=LED4_OF;
		*/
		
		if (FlagStepRA){ //     RA 
			FlagStepRA = 0;
			encoderGoto(&RA); //     GoTo
		}
		if (FlagStepDEC){ //     DEC 
			FlagStepDEC = 0;
			encoderGoto(&DEC);
		}
		
		//          199
		slowStartStopRA(&RA); 
		slowStartStopDEC(&DEC);
		
		if (FlagRxEnd){ //      USART
			FlagRxEnd = 0;
			GPIOC->BSRR = LED4_ON; //  LED4
			if(!(eqmod(RxBuffer, RxBuffer, &RA, &DEC))) //   
			{
				//       
				initSpeedRA(&RA);
				initSpeedDEC(&DEC);
			}
			GPIOC->BSRR = LED4_OF; //  LED4
		}
		
		if (RxCounter){ //     USART1
			if(FlagResetTimeOut){
				FlagResetTimeOut = 0;
				TimeOut = SysTickTime + 10000; // Timeout = 10 c.
			}
			if(TimeOut <= SysTickTime) RxCounter = 0;
		}
		
	}
}

void SysTick_Handler (void){
	SysTickTime++;
}

void EXTI0_IRQHandler (void){
    //GPIOC->ODR ^= LED3;
    EXTI->PR = EXTI_PR_PR0;
    //NVIC_ClearPendingIRQ (EXTI0_IRQn);
}

void TIM1_UP_TIM16_IRQHandler (void){
	if (DEC.StatusON) {STEP_DEC_UP; FlagStepDEC =1;}
    TIM16->SR &= ~TIM_SR_UIF;
    //NVIC_ClearPendingIRQ (TIM1_UP_TIM16_IRQn);
	STEP_DEC_DW;
}

void TIM1_TRG_COM_TIM17_IRQHandler (void){
	GPIOC->ODR ^= LED3;
	if (RA.StatusON) {STEP_RA_UP; FlagStepRA =1;}
    TIM17->SR &= ~TIM_SR_UIF;
    //NVIC_ClearPendingIRQ (TIM1_TRG_COM_TIM17_IRQn);
	STEP_RA_DW;
}

void USART1_IRQHandler (void){ //    USART1
u32 UsartData, UsartState;

	UsartState = USART1->SR;
	UsartData  = USART1->DR; //    SR  DR 
	if (UsartState & USART_SR_RXNE){ //    
		if (!(UsartState & 0x0007)){ //    
			FlagResetTimeOut = 1; //  
			if (!RxCounter){ //    
				if (UsartData == ':') RxCounter++;
			}
			else{
				if (UsartData == '\r'){ //    
					RxBuffer[0] = RxCounter - 1;
					RxCounter = 0;
					FlagRxEnd = 1;
				}
				else RxBuffer[RxCounter++] = UsartData;
			}
		}
	}
	if (UsartState & USART_SR_TC){ //     .
		if(TxCounter){ //       
			TxCounter--;
			USART1->DR = TxBuffer[TxRdIndex++];
			if(TxRdIndex > 10) TxRdIndex = 0;
		}
		USART1->SR &= ~USART_SR_TC; //   " ".
	}
}
